<?php

namespace App\Factory;

use App\Exception;

class PriceFactory {

    private $connection;
    
    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }

    public function clientBranchPricingTable($CDCLIENTE, $CDFILIAL) {
        $sql = "
    		SELECT CDTABEPREC, CDCFILTABPRE AS CDFILTABPREC
    		  FROM CLIENFILIAL
    		 WHERE CDCLIENTE = :CDCLIENTE
    		   AND CDFILIAL  = :CDFILIAL
	    ";
        
        $params = [
            ':CDCLIENTE' => $CDCLIENTE,
            ':CDFILIAL' => $CDFILIAL
        ];

        $result = $this->connection->fetchAll($sql, $params);
        return (!empty($result[0]['CDTABEPREC']) ? $result[0] : []);
    }

    public function clientPricingTable($CDCLIENTE) {
        $sql = "
    		SELECT CDTABEPREC, CDFILTABPREC AS CDFILTABPREC
    		  FROM CLIENTE
    		 WHERE CDCLIENTE = :CDCLIENTE
    	";
        
        $params = [
            ':CDCLIENTE' => $CDCLIENTE
        ];

        $result = $this->connection->fetchAll($sql, $params);
        return (!empty($result[0]['CDTABEPREC']) ? $result[0] : []);
    }

    public function storePricingTable($CDFILIAL, $CDLOJA) {
        $sql = "
    		SELECT CDTABEPREC, NULL AS CDFILTABPREC
    		  FROM LOJA
    		 WHERE CDFILIAL = :CDFILIAL
    		   AND CDLOJA   = :CDLOJA
    	";
        
        $params = [
            ':CDFILIAL' => $CDFILIAL,
            ':CDLOJA' => $CDLOJA
        ];

        $result = $this->connection->fetchAll($sql, $params);
        return (!empty($result[0]['CDTABEPREC']) ? $result[0] : []);
    }

    public function paravendPricingTable($CDFILIAL){
        $sql = "
    		SELECT NVL(CDTABEPREC, CDTABEPRECDLV) AS CDTABEPREC, NULL AS CDFILTABPREC
    		  FROM PARAVEND
    		 WHERE CDFILIAL = :CDFILIAL
    	";
    	
        $params = [
            ':CDFILIAL' => $CDFILIAL
        ];

        $result = $this->connection->fetchAll($sql, $params);
        return (!empty($result[0]['CDTABEPREC']) ? $result[0] : []);
    }

    public function getSalesTable($CDFILIAL, $CDTABEPREC){
        $sql = "
    		SELECT TO_CHAR(DTINIVGPREC, 'DD/MM/YYYY') AS DTINIVGPREC, DTINIVGPREC AS ORIGINAL
    		  FROM TABEVEND
    		 WHERE CDFILIAL   = :CDFILIAL
    		   AND CDTABEPREC = :CDTABEPREC
    		   AND (SYSDATE BETWEEN DTINIVGPREC AND DTFINVGPREC)
    	";
        
        $params = [
            ':CDFILIAL' => $CDFILIAL,
            ':CDTABEPREC' => $CDTABEPREC
        ];

        $result = $this->connection->fetchAll($sql, $params);
        return (!empty($result) ? $result[0] : $result);
    }
    
    public function getGeneralPricingTableValidity($CDTABEPREC){
        $sql = "
    		SELECT TO_CHAR(DTINIVGPREC, 'DD/MM/YYYY') AS DTINIVGPREC
    		  FROM TABEVENDGER
    		 WHERE CDTABEPREC = :CDTABEPREC
    		   AND (SYSDATE BETWEEN DTINIVGPREC AND DTFINVGPREC)
    	";
        
        $params = [
            ':CDTABEPREC' => $CDTABEPREC
        ];

        $result = $this->connection->fetchAll($sql, $params);
        return (!empty($result) ? $result[0] : $result);
    }

    public function holidayCheck($CDFILIAL, $DTFERIFILI){
        $sql = "
    		SELECT DTFERIFILI
    		  FROM FERIFILI
    		 WHERE CDFILIAL   = :CDFILIAL
    		   AND DTFERIFILI = :DTFERIFILI
    	";
        
        $params = [
            ':CDFILIAL' => $CDFILIAL,
            ':DTFERIFILI' => $DTFERIFILI
        ];

        $result = $this->connection->fetchAll($sql, $params);
        return (!empty($result) ? $result[0] : $result);
    }

    public function getProductDiscountType ($CDFILIAL, $CDTIPOCONS) {
        $sql = "
    		SELECT DISTINCT I.CDPRODUTO, I.IDPERVALORPR TIPO
    		    FROM ITEMPRECODIA I 
    		
    		    LEFT JOIN TIPOCONS T 
    		        ON (I.CDTIPOCONSPD = T.CDTIPOCONS) 
    		    LEFT JOIN TIPOCONSFILI TF 
    		        ON((TF.CDFILIAL = :CDFILIAL) AND (TF.CDTIPOCONS = I.CDTIPOCONSPD))
    		    
    		    WHERE T.CDTIPOCONS IS NULL
    		       OR T.CDTIPOCONS = :CDTIPOCONS
    	";
        
        $params = [
            ':CDFILIAL'   => $CDFILIAL,
            ':CDTIPOCONS' => $CDTIPOCONS
        ];
        
        return $this->connection->fetchAll($sql, $params);
    }

    public function getPricesOnHoliday($CDFILIAL, $CDTABEPREC, $DTINIVGPREC, $NRDIAATUAL){
        $sql = "
    		SELECT	I.CDPRODUTO, I.VRPRECODIA, 
    				I.CDTABEPREC, I.DTINIVGPREC, 
    				I.IDPERVALORPR, I.IDDESCACREPR
    		  
    		  FROM ITEMPRECODIA I
    	  	 
    	  	 WHERE I.CDFILIAL	 = :CDFILIAL
      		   AND I.CDTABEPREC  = :CDTABEPREC
      		   AND I.DTINIVGPREC = TO_DATE(:DTINIVGPREC, 'DD/MM/YYYY')
      		   AND (I.NRDIASEMANPR = :NRDIAATUAL OR I.NRDIASEMANPR = 'F')
      		   AND TO_CHAR(SYSDATE,'HH24')||'00' BETWEEN I.HRINIPRECDIA AND I.HRFINPRECDIA
    	";
        
        $params = [
            ':CDFILIAL'     => $CDFILIAL,
            ':CDTABEPREC'   => $CDTABEPREC,
            ':DTINIVGPREC'  => $DTINIVGPREC,
            ':NRDIAATUAL'   => $NRDIAATUAL
        ];

        return $this->connection->fetchAll($sql, $params);   
    }

    public function getPricesByDay($CDFILIAL, $CDTABEPREC, $DTINIVGPREC, $NRDIAATUAL, $CDTIPOCONSPD){
        $sql = "
        	SELECT CDPRODUTO, VRPRECODIA, IDPERVALORPR, IDDESCACREPR
              FROM ITEMPRECODIA
             WHERE CDFILIAL = :CDFILIAL
               AND CDTABEPREC = :CDTABEPREC
               AND (NRDIASEMANPR = :NRDIAATUAL OR NRDIASEMANPR = 'T')
               AND TRUNC(SYSDATE) BETWEEN TRUNC(DTINIVALPREC) AND TRUNC(DTFINVALPREC)
               AND (CDTIPOCONSPD = :CDTIPOCONSPD OR CDTIPOCONSPD = 'T')
               AND TO_CHAR(SYSDATE,'HH24')||'00' BETWEEN HRINIPRECDIA AND HRFINPRECDIA
    	";
        
        $params = [
            ':CDFILIAL'     => $CDFILIAL,
            ':CDTABEPREC'   => $CDTABEPREC,
            ':DTINIVGPREC'  => $DTINIVGPREC,
            ':NRDIAATUAL'   => $NRDIAATUAL,
            ':CDTIPOCONSPD' => $CDTIPOCONSPD
        ];

        return $this->connection->fetchAll($sql, $params);   
    }

    public function getPrices($CDFILIAL, $CDFILTABPREC, $CDTABEPREC, $DTINIVGPREC){
        $sql = "
    		SELECT I.CDPRODUTO, I.VRPRECITEM, I.CDTABEPREC, I.DTINIVGPREC, P.IDIMPPRODUTO, 
    			I.VRPRECITEM AS FULLPRICE
    		  
    		  FROM ITEMPRECO I
          	  
          	  JOIN PRODUTO P
            	ON I.CDPRODUTO = P.CDPRODUTO
    		
    		 WHERE (I.CDFILIAL = :CDFILIAL OR I.CDFILIAL = :CDFILTABPREC)
    		   AND I.CDTABEPREC = :CDTABEPREC
    		   AND TRUNC(I.DTINIVGPREC) >= TO_DATE(:DTINIVGPREC, 'DD/MM/YYYY')
    		   
    	  ORDER BY I.CDPRODUTO
    	";
        
        $params = [
            ':CDFILIAL'     => $CDFILIAL,
            ':CDTABEPREC'   => $CDTABEPREC,
            ':DTINIVGPREC'  => $DTINIVGPREC,
            ':CDFILTABPREC' => $CDFILTABPREC
        ];
        
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getGeneralPrices($CDTABEPREC, $DTINIVGPREC){
        $sql = "
    		SELECT I.CDPRODUTO, I.VRPRECITEM, I.CDTABEPREC, I.DTINIVGPREC, P.IDIMPPRODUTO, 
    			I.VRPRECITEM AS FULLPRICE, P.NMPRODUTO
    		  
    		  FROM ITEMPRECOGER I
          	  
          	  JOIN PRODUTO P
            	ON I.CDPRODUTO = P.CDPRODUTO
    		
    		 WHERE I.CDTABEPREC = :CDTABEPREC
    		   AND TRUNC(I.DTINIVGPREC) >= TO_DATE(:DTINIVGPREC, 'DD/MM/YYYY')
    		   
    	  ORDER BY I.CDPRODUTO
    	";
        
        $params = [
            ':CDTABEPREC'   => $CDTABEPREC,
            ':DTINIVGPREC'  => $DTINIVGPREC
        ];

        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDiffPrices($CDTABEPREC, $NRDIASEMANPR){
        $sql = "
    		SELECT CDPRODUTO, VRPRECODIA
              FROM ITEMPRECODIAGER
             WHERE (NRDIASEMANPR = 'T' OR NRDIASEMANPR = :NRDIASEMANPR)
    	";
        
        $params = [
            ':CDTABEPREC'   => $CDTABEPREC,
            ':NRDIASEMANPR'  => $NRDIASEMANPR
        ];

        return $this->connection->fetchAll($sql, $params);
    }

    public function getItemPrice($CDFILIAL, $CDTABEPREC, $DATAVIG, $CDPRODUTO){
        $sql = "
    		SELECT NVL(VRPRECITEM  ,0) AS FULLPRICE,
    			   NVL(VRPRECITEMCL,0) AS PRECOCLIE,
    			   NVL(VRPRESUGITEM,0) AS PRECOSUGER, IDPRECVARIA
    		  FROM ITEMPRECO
    		 WHERE CDFILIAL    = :CDFILIAL
    		   AND CDTABEPREC  = :CDTABEPREC
    		   AND DTINIVGPREC = TO_DATE(:DATAVIG,'DD/MM/YYYY')
    		   AND CDPRODUTO   = :CDPRODUTO
    		ORDER BY CDPRODUTO
    	";
        
        $params = [
            ':CDFILIAL' => $CDFILIAL,
            ':CDTABEPREC' => $CDTABEPREC,
            ':DATAVIG' => $DATAVIG,
            ':CDPRODUTO' => $CDPRODUTO
        ];

        $result = $this->connection->fetchAll($sql, $params);
        return (!empty($result) ? $result[0] : $result);
    }
    
    public function getGeneralPricingTable($CDFILIAL){
        $sql = "
    		SELECT CDTABEPREC
    		  FROM FILITABGER
    		 WHERE CDFILIAL = :CDFILIAL
    	";
        
        $params = [
            ':CDFILIAL' => $CDFILIAL
        ];

        return $this->connection->fetchAssoc($sql, $params);   
    }
}